/* See LICENSE for licensing and NOTICE for copyright. */
package org.ldaptive.pool;

/**
 * Enum to define queue type.
 *
 * @author  Middleware Services
 */
public enum QueueType {

  /** first in, first out ordering. */
  FIFO,

  /** last in, first out ordering. */
  LIFO
}
