/* See LICENSE for licensing and NOTICE for copyright. */
package org.ldaptive.sasl;

/**
 * Enum to define SASL security strength.
 *
 * @author  Middleware Services
 */
public enum SecurityStrength {

  /** High security strength. */
  HIGH,

  /** Medium security strength. */
  MEDIUM,

  /** Low security strength. */
  LOW
}
